==> tgam-patterns-2.1.2.tgz: package/patterns/javascripts/panel.js <==
/**
 * js logic specific to opening and closing the panels from the site header elements
 */

import { focusElementInPanel, ariaToggle, toArray } from "~/dom-methods";
import ee from "~/event-emitter";

const panelSelector = ".o-panel";
const panelCatSelector = ".o-panel-categories";
const panelSearchSelector = ".o-panel-search";
const panelBgHitSelector = ".o-panel-bg-area";
const menuButtonSelector = ".c-control--menu-button";
const searchButtonSelector = ".c-control--search-button";
const closeControlSelector = ".c-control--close";
const panelOpenClass = "o-panel--open";
const panelCatsOpenClass = "o-panel-categories--open";
const panelSearchOpenClass = "o-panel-search--open";
const animationTime = 500;

let menuButton = null;
let panel = null;
let panelCategories = null;
let panelSearch = null;
let panelBgHit = null;
let closeControl = null;
let searchButton = null;
let classSwapTimeout = null;

/**
 * @method init
 * @description init method, adds event listener for scroll and resize
 * @return {void}
 */
export default function init() {
  panel = document.querySelector(panelSelector);
  panelCategories = document.querySelector(panelCatSelector);
  panelSearch = document.querySelector(panelSearchSelector);
  menuButton = toArray(document.querySelectorAll(menuButtonSelector));
  searchButton = toArray(document.querySelectorAll(searchButtonSelector));
  closeControl = toArray(document.querySelectorAll(closeControlSelector));

  menuButton.forEach((el) => {
    el.addEventListener("click", onCategoriesClicked);
  });

  closeControl.forEach((el) => {
    el.addEventListener("click", panelClose);
  });

  searchButton.forEach((el) => {
    el.addEventListener("click", onSearchClicked);
  });
}

/**
 * @method onCategoriesClicked
 * @description on menu clicked
 * @return {void}
 */
export function onCategoriesClicked() {
  const isMenuOpen = panel.classList.contains(panelOpenClass);
  const isCatOpen = panelCategories.classList.contains(panelCatsOpenClass);

  isMenuOpen && isCatOpen ? panelClose() : categoriesOpen();
}

/**
 * @method categoriesOpen
 * @description adds open classes
 * @return {void}
 */
export function categoriesOpen() {
  panel.classList.add(panelOpenClass);
  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    panelCategories.classList.add(panelCatsOpenClass);
    panelSearch.classList.remove(panelSearchOpenClass);

    // if first element return false, then focus the second element
    !focusElementInPanel(".c-search__input") && focusElementInPanel(".c-accordion__item .c-control__button");
    ee.emit("panel:categories-opened");
    ee.emit("panel:opened");
  });

  ariaToggle(".c-control--menu-button .c-control__button", "close navigation", true);
  navigationFade();
  addBgPanel();
  ee.emit("panel:categories-opening");
  ee.emit("panel:opening");
}

/**
 * @method onSearchClicked
 * @description adds open class
 * @return {void}
 */
export function onSearchClicked() {
  const isMenuOpen = panel.classList.contains(panelOpenClass);
  const isSearchOpen = panelSearch.classList.contains(panelSearchOpenClass);

  isMenuOpen && isSearchOpen ? panelClose() : searchOpen();
}

/**
 * @method searchOpen
 * @description adds open class
 * @return {void}
 */
export function searchOpen() {
  panel.classList.add(panelOpenClass);
  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    panelSearch.classList.add(panelSearchOpenClass);
    panelCategories.classList.remove(panelCatsOpenClass);

    // if first element return false, then focus the second element
    !focusElementInPanel(".c-search__input") && focusElementInPanel(".c-accordion__item__link");
    ee.emit("panel:search-opened");
    ee.emit("panel:opened");
  }, animationTime);

  ariaToggle(".c-control--search-button .c-control__button", "close search", true);
  navigationFade();
  addBgPanel();
  ee.emit("panel:search-opening");
  ee.emit("panel:opening");
}

/**
 * @method panelClose
 * @description removes open classes
 * @return {void}
 */
export function panelClose() {
  panel.classList.remove(panelOpenClass);
  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    panelCategories.classList.remove(panelCatsOpenClass);
    panelSearch.classList.remove(panelSearchOpenClass);

    ee.emit("panel:closed");
  }, animationTime);

  ariaToggle(".c-control--menu-button .c-control__button", "open navigation", false);
  ariaToggle(".c-control--search-button .c-control__button", "open search", false);
  navigationUnFade();
  removeBgPanel();

  let focusElements = toArray(document.querySelectorAll(".c-control--menu-button .c-control__button"));

  // loop through 2 button element to focus on, whichever one is visible will be focused
  focusElements.some((m) => {
    let isVisible = m.offsetHeight !== 0;
    if (isVisible) {
      m.focus();
    }
    return isVisible;
  });
  ee.emit("panel:closing");
}

/**
 * @method addBgPanel
 * @description adds the hidden background element
 * @return {void}
 */
export function addBgPanel() {
  if (!document.querySelector(".o-panel-bg-area")) {
    const divPanelBG = document.createElement("div");
    divPanelBG.classList.add("o-panel-bg-area");
    document.body.appendChild(divPanelBG);
    panelBgHit = document.querySelector(panelBgHitSelector);
    panelBgHit.addEventListener("click", panelClose);
  } else {
    document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.remove("o-panel-bg-area--closed");
  }
}

/**
 * @method removeBgPanel
 * @description removes the hidden background element
 * @return {void}
 */
export function removeBgPanel() {
  document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.add("o-panel-bg-area--closed");
}

/**
 * @method navigationFade
 * @description adds fade class to specific elements
 * @return {void}
 */
export function navigationFade() {
  const navBar = document.querySelector(".c-header-title .c-scroll-menu__nav");
  const h2 = document.querySelector("h2");
  const ignoreClasses = ["c-control--menu-button", "c-control--search-button"];
  const headerElement = document.querySelector(".c-header-nav") ||
    document.querySelector(".c-header-title--article") ||
    document.querySelector(".c-header-title--section") ||
    document.querySelector(".c-header-title--variant");
  const allControlButtons = toArray(headerElement.querySelectorAll(".c-control"));
  const fadeClass = "u-faded";

  allControlButtons.forEach((c) => {
    let notAnIgnoredClass = ignoreClasses.every((ignoreString) => {
      return !c.classList.contains(ignoreString);
    });

    notAnIgnoredClass && c.classList.add(fadeClass);
  });

  navBar && navBar.classList.add(fadeClass);
  h2 && h2.classList.add(fadeClass);
}

/**
 * @method navigationUnFade
 * @description removes fade class to specific elements
 * @return {void}
 */
export function navigationUnFade() {
  const navBar = document.querySelector(".c-header-title .c-scroll-menu__nav");
  const h2 = document.querySelector("h2");
  const headerElement = document.querySelector(".c-header-nav") ||
    document.querySelector(".c-header-title--article") ||
    document.querySelector(".c-header-title--section") ||
    document.querySelector(".c-header-title--variant");
  const allControlButtons = toArray(headerElement.querySelectorAll(".c-control"));
  const fadeClass = "u-faded";

  allControlButtons.forEach((c) => {
    c.classList.remove(fadeClass);
  });

  navBar && navBar.classList.remove(fadeClass);
  h2 && h2.classList.remove(fadeClass);
}

==> tgam-patterns-2.1.12.tgz: package/patterns/javascripts/panel.js <==
/**
 * js logic specific to opening and closing the panels from the site header elements
 */

import { ariaToggle, toArray } from "~/dom-methods";
import ee from "~/event-emitter";

const panelSelector = "#o-panel";
const panelCatSelector = ".o-panel-categories";
const panelSearchSelector = ".o-panel-search";
const panelBgHitSelector = ".o-panel-bg-area";
const menuButtonSelector = ".c-control--menu-button";
const searchButtonSelector = ".c-control--search-button";
const closeControlSelector = ".c-control--close";
const panelOpenClass = "o-panel--open";
const panelCatsOpenClass = "o-panel-categories--open";
const panelSearchOpenClass = "o-panel-search--open";
const animationTime = 250;
const delayClassSwitchTime = 50;

let menuButton = null;
let panel = null;
let panelCategories = null;
let panelSearch = null;
let panelBgHit = null;
let closeControl = null;
let searchButton = null;
let classSwapTimeout = null;
let delayTimeout = null;

/**
 * @method init
 * @description init method, adds event listener for scroll and resize
 * @return {void}
 */
export default function init() {
  panel = document.querySelector(panelSelector);
  panelCategories = document.querySelector(panelCatSelector);
  panelSearch = document.querySelector(panelSearchSelector);
  menuButton = toArray(document.querySelectorAll(menuButtonSelector));
  searchButton = toArray(document.querySelectorAll(searchButtonSelector));
  closeControl = toArray(document.querySelectorAll(closeControlSelector));

  menuButton.forEach((el) => {
    el.addEventListener("click", onCategoriesClicked);
  });

  closeControl.forEach((el) => {
    el.addEventListener("click", panelClose);
  });

  searchButton.forEach((el) => {
    el.addEventListener("click", onSearchClicked);
  });
}

/**
 * @method onCategoriesClicked
 * @description on menu clicked
 * @return {void}
 */
export function onCategoriesClicked() {
  const isMenuOpen = panel.classList.contains(panelOpenClass);
  const isCatOpen = panelCategories.classList.contains(panelCatsOpenClass);

  isMenuOpen && isCatOpen ? panelClose() : categoriesOpen();
}

/**
 * @method onSearchClicked
 * @description adds open class
 * @return {void}
 */
export function onSearchClicked() {
  const isMenuOpen = panel.classList.contains(panelOpenClass);
  const isSearchOpen = panelSearch.classList.contains(panelSearchOpenClass);

  isMenuOpen && isSearchOpen ? panelClose() : searchOpen();
}

/**
 * @method categoriesOpen
 * @description adds open classes
 * @return {void}
 */
export function categoriesOpen() {
  panel.removeAttribute("hidden");

  clearTimeout(delayTimeout);
  delayTimeout = setTimeout(() => {
    panel.classList.add(panelOpenClass);
    panelCategories.classList.add(panelCatsOpenClass);
    panelSearch.classList.remove(panelSearchOpenClass);

    ariaToggle(".c-control--menu-button .c-control__button", "close navigation", true);
    navigationFade();
    addBgPanel();

    ee.emit("panel:categories-opening");
    ee.emit("panel:opening");
  }, delayClassSwitchTime);

  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    ee.emit("panel:categories-opened");
    ee.emit("panel:opened");
  }, animationTime + delayClassSwitchTime);
}

/**
 * @method searchOpen
 * @description adds open class
 * @return {void}
 */
export function searchOpen() {
  panel.removeAttribute("hidden");

  clearTimeout(delayTimeout);
  delayTimeout = setTimeout(() => {
    panel.classList.add(panelOpenClass);
    panelSearch.classList.add(panelSearchOpenClass);
    panelCategories.classList.remove(panelCatsOpenClass);

    ariaToggle(".c-control--search-button .c-control__button", "close search", true);
    navigationFade();
    addBgPanel();

    ee.emit("panel:search-opening");
    ee.emit("panel:opening");
  }, delayClassSwitchTime);

  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    ee.emit("panel:search-opened");
    ee.emit("panel:opened");
  }, animationTime + delayClassSwitchTime);
}

/**
 * @method panelClose
 * @description removes open classes
 * @return {void}
 */
export function panelClose() {
  panel.classList.remove(panelOpenClass);

  ariaToggle(".c-control--menu-button .c-control__button", "open navigation", false);
  ariaToggle(".c-control--search-button .c-control__button", "open search", false);
  navigationUnFade();
  removeBgPanel();
  ee.emit("panel:closing");
  ee.emit("panel:search-closing");
  ee.emit("panel:categories-closing");

  clearTimeout(classSwapTimeout);
  classSwapTimeout = setTimeout(() => {
    panelCategories.classList.remove(panelCatsOpenClass);
    panelSearch.classList.remove(panelSearchOpenClass);
    panel.setAttribute("hidden", "hidden");

    ee.emit("panel:closed");
    ee.emit("panel:search-closed");
    ee.emit("panel:categories-closed");
  }, animationTime);
}

/**
 * @method addBgPanel
 * @description adds the hidden background element
 * @return {void}
 */
export function addBgPanel() {
  if (!document.querySelector(".o-panel-bg-area")) {
    const divPanelBG = document.createElement("div");
    divPanelBG.classList.add("o-panel-bg-area");
    document.body.appendChild(divPanelBG);
    panelBgHit = document.querySelector(panelBgHitSelector);
    panelBgHit.addEventListener("click", panelClose);
  } else {
    document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.remove("o-panel-bg-area--closed");
  }
}

/**
 * @method removeBgPanel
 * @description removes the hidden background element
 * @return {void}
 */
export function removeBgPanel() {
  document.querySelector(panelBgHitSelector) && document.querySelector(panelBgHitSelector).classList.add("o-panel-bg-area--closed");
}

/**
 * @method getHeaderElement
 * @description gets the current header element
 * @return {HTMLElement}
 */
export function getHeaderElement() {
  return document.querySelector(".c-header-nav") || document.querySelector(".c-header-title--article") || document.querySelector(".c-header-title--section");
}

/**
 * @method navigationFade
 * @description adds fade class to specific elements
 * @return {void}
 */
export function navigationFade() {
  const fadeClass = "c-control--disabled";
  const h2 = document.querySelector("h2");
  const scrollNav = document.querySelector(".c-scroll-menu__nav");
  const header = getHeaderElement();
  const allControlButtons = toArray(header.querySelectorAll(".c-control"));
  const ignoreClasses = ["c-control--menu-button", "c-control--search-button"];

  allControlButtons.forEach((c) => {
    let notAnIgnoredClass = ignoreClasses.every((ignoreString) => {
      return !c.classList.contains(ignoreString);
    });

    notAnIgnoredClass && c.classList.add(fadeClass);
  });

  scrollNav && scrollNav.classList.add(fadeClass);
  h2 && h2.classList.add(fadeClass);
}

/**
 * @method navigationUnFade
 * @description removes fade class to specific elements
 * @return {void}
 */
export function navigationUnFade() {
  const fadeClass = "c-control--disabled";
  const h2 = document.querySelector("h2");
  const scrollNav = document.querySelector(".c-scroll-menu__nav");
  const header = getHeaderElement();
  const allControlButtons = toArray(header.querySelectorAll(".c-control"));

  allControlButtons.forEach((c) => {
    c.classList.remove(fadeClass);
  });

  scrollNav && scrollNav.classList.remove(fadeClass);
  h2 && h2.classList.remove(fadeClass);
}
